@@ -14,6 +14,7 @@ from flask import flash
 from flask import redirect
 from flask import render_template
 from flask import request
+from flask import session
 from flask import url_for
 from flask_peewee.filters import FilterForm
 from flask_peewee.filters import FilterMapping
@@ -180,7 +181,7 @@ class ModelAdmin(object):
         )
 
     def get_columns(self):
-        return self.model._meta.get_field_names()
+        return self.model._meta.sorted_field_names
 
     def column_is_sortable(self, col):
         return col in self.model._meta.fields
@@ -208,6 +209,7 @@ class ModelAdmin(object):
         return {}
 
     def index(self):
+        session['%s.index' % self.get_admin_name()] = request.url
         query = self.get_query()
 
         ordering = request.args.get('ordering') or ''
@@ -238,7 +240,9 @@ class ModelAdmin(object):
 
     def dispatch_save_redirect(self, instance):
         if 'save' in request.form:
-            return redirect(url_for(self.get_url_name('index')))
+            url = (session.get('%s.index' % self.get_admin_name()) or
+                   url_for(self.get_url_name('index')))
+            return redirect(url)
         elif 'save_add' in request.form:
             return redirect(url_for(self.get_url_name('add')))
         else:
@@ -323,7 +327,9 @@ class ModelAdmin(object):
                 obj.delete_instance(recursive=self.delete_recursive)
 
             flash('Successfully deleted %s %ss' % (count, self.get_display_name()), 'success')
-            return redirect(url_for(self.get_url_name('index')))
+            url = (session.get('%s.index' % self.get_admin_name()) or
+                   url_for(self.get_url_name('index')))
+            return redirect(url)
 
         return render_template(self.templates['delete'], **dict(
             model_admin=self,
@@ -335,7 +341,7 @@ class ModelAdmin(object):
     def collect_related_fields(self, model, accum, path, seen=None):
         seen = seen or set()
         path_str = '__'.join(path)
-        for field in model._meta.get_fields():
+        for field in model._meta.sorted_fields:
             if isinstance(field, ForeignKeyField) and field not in seen:
                 seen.add(field)
                 self.collect_related_fields(field.rel_model, accum, path + [field.name], seen)
@@ -451,10 +457,21 @@ class AdminTemplateHelper(object):
         self.app = self.admin.app
 
     def get_model_field(self, model, field):
-        attr = getattr(model, field)
-        if callable(attr):
-            return attr()
-        return attr
+        try:
+            attr = getattr(model, field)
+        except AttributeError:
+            model_admin = self.admin[type(model)]
+            try:
+                attr = getattr(model_admin, field)
+            except AttributeError:
+                raise AttributeError('Could not find attribute or method '
+                                     'named "%s".' % field)
+            else:
+                return attr(model)
+        else:
+            if callable(attr):
+                attr = attr()
+            return attr
 
     def get_form_field(self, form, field_name):
         return getattr(form, field_name)
