@@ -1,11 +1,24 @@
 import functools
 import os
 
-from flask import Blueprint, render_template, abort, request, session, flash, redirect, url_for, g
+from flask import Blueprint
+from flask import abort
+from flask import flash
+from flask import g
+from flask import redirect
+from flask import render_template
+from flask import request
+from flask import session
+from flask import url_for
 from peewee import *
-from wtforms import Form, TextField, PasswordField, validators
+from wtforms import Form
+from wtforms import PasswordField
+from wtforms import TextField
+from wtforms import validators
 
-from flask_peewee.utils import get_next, make_password, check_password
+from flask_peewee.utils import check_password
+from flask_peewee.utils import get_next
+from flask_peewee.utils import make_password
 
 
 current_dir = os.path.dirname(__file__)
@@ -27,10 +40,11 @@ class BaseUser(object):
 
 class Auth(object):
     def __init__(self, app, db, user_model=None, prefix='/accounts', name='auth',
-                 clear_session=False, default_next_url='/'):
+                 clear_session=False, default_next_url='/', db_table='user'):
         self.app = app
         self.db = db
 
+        self.db_table = db_table
         self.User = user_model or self.get_user_model()
 
         self.blueprint = self.get_blueprint(name)
@@ -55,6 +69,9 @@ class Auth(object):
             def __unicode__(self):
                 return self.username
 
+            class Meta:
+                db_table = self.db_table  # Postgres reserves user as a keyword
+
         return User
 
     def get_model_admin(self, model_admin=None):
@@ -63,7 +80,8 @@ class Auth(object):
             model_admin = ModelAdmin
 
         class UserAdmin(model_admin):
-            columns = ['username', 'email', 'active', 'admin']
+            columns = getattr(model_admin, 'columns') or (
+                    ['username', 'email', 'active', 'admin'])
 
             def save_model(self, instance, form, adding=False):
                 orig_password = instance.password
@@ -104,7 +122,7 @@ class Auth(object):
             @functools.wraps(fn)
             def inner(*args, **kwargs):
                 user = self.get_logged_in_user()
-                
+
                 if not user or not test_fn(user):
                     login_url = url_for('%s.login' % self.blueprint.name, next=get_next())
                     return redirect(login_url)
@@ -114,7 +132,7 @@ class Auth(object):
 
     def login_required(self, func):
         return self.test_user(lambda u: True)(func)
-    
+
     def admin_required(self, func):
         return self.test_user(lambda u: u.admin)(func)
 
@@ -135,9 +153,9 @@ class Auth(object):
         session['user_pk'] = user.get_id()
         session.permanent = True
         g.user = user
-        flash('You are logged in as %s' % user.username, 'success')
+        flash('You are logged in as %s' % user, 'success')
 
-    def logout_user(self, user):
+    def logout_user(self):
         if self.clear_session:
             session.clear()
         else:
@@ -152,7 +170,7 @@ class Auth(object):
 
             try:
                 return self.User.select().where(
-                    self.User.active==True, 
+                    self.User.active==True,
                     self.User.id==session.get('user_pk')
                 ).get()
             except self.User.DoesNotExist:
@@ -164,6 +182,7 @@ class Auth(object):
 
         if request.method == 'POST':
             form = Form(request.form)
+            next_url = request.form.get('next') or self.default_next_url
             if form.validate():
                 authenticated_user = self.authenticate(
                     form.username.data,
@@ -171,23 +190,23 @@ class Auth(object):
                 )
                 if authenticated_user:
                     self.login_user(authenticated_user)
-                    return redirect(
-                        request.args.get('next') or \
-                        self.default_next_url
-                    )
+                    return redirect(next_url)
                 else:
                     flash('Incorrect username or password')
         else:
             form = Form()
+            next_url = request.args.get('next')
 
-        return render_template('auth/login.html', error=error, form=form)
+        return render_template(
+            'auth/login.html',
+            error=error,
+            form=form,
+            login_url=url_for('%s.login' % self.blueprint.name),
+            next=next_url)
 
     def logout(self):
-        self.logout_user(self.get_logged_in_user())
-        return redirect(
-            request.args.get('next') or \
-            self.default_next_url
-        )
+        self.logout_user()
+        return redirect(request.args.get('next') or self.default_next_url)
 
     def configure_routes(self):
         for url, callback in self.get_urls():
