@@ -2,12 +2,27 @@ from __future__ import with_statement
 
 import datetime
 
-from flask import request, session, url_for, g
+from flask import g
+from flask import request
+from flask import session
+from flask import url_for
 
-from flask_peewee.admin import ModelAdmin, AdminPanel
+from flask_peewee.admin import AdminPanel
+from flask_peewee.admin import ModelAdmin
 from flask_peewee.tests.base import FlaskPeeweeTestCase
-from flask_peewee.tests.test_app import User, Message, Note, admin, AModel, BModel, CModel, DModel, BDetails
-from flask_peewee.utils import get_next, make_password, check_password
+from flask_peewee.tests.test_app import AModel
+from flask_peewee.tests.test_app import BDetails
+from flask_peewee.tests.test_app import BModel
+from flask_peewee.tests.test_app import CModel
+from flask_peewee.tests.test_app import DModel
+from flask_peewee.tests.test_app import Message
+from flask_peewee.tests.test_app import Note
+from flask_peewee.tests.test_app import User
+from flask_peewee.tests.test_app import admin
+from flask_peewee.utils import check_password
+from flask_peewee.utils import get_next
+from flask_peewee.utils import make_password
+from flask_peewee._compat import text_type
 
 from wtfpeewee.orm import model_form
 
@@ -35,9 +50,10 @@ class AdminTestCase(BaseAdminTestCase):
         self.assertEqual(resp.headers['location'], 'http://localhost/accounts/login/?next=%2Fadmin%2F')
 
         # try logging in as a normal user, get a 403 forbidden
-        resp = self.app.post('/accounts/login/?next=%2Fadmin%2F', data={
+        resp = self.app.post('/accounts/login/', data={
             'username': 'normal',
             'password': 'normal',
+            'next': '/admin/',
         })
         self.assertEqual(resp.status_code, 302)
         self.assertEqual(resp.headers['location'], 'http://localhost/admin/')
@@ -49,9 +65,10 @@ class AdminTestCase(BaseAdminTestCase):
         resp = self.app.get('/accounts/logout/')
 
         # try logging in as an admin and get a 200
-        resp = self.app.post('/accounts/login/?next=%2Fadmin%2F', data={
+        resp = self.app.post('/accounts/login/', data={
             'username': 'admin',
             'password': 'admin',
+            'next': '/admin/',
         })
         self.assertEqual(resp.status_code, 302)
         self.assertEqual(resp.headers['location'], 'http://localhost/admin/')
@@ -688,12 +705,13 @@ class TemplateHelperTestCase(FlaskPeeweeTestCase):
         self.assertEqual(self.th.fix_underscores('test'), 'Test')
 
     def test_update_querystring(self):
-        self.assertEqual(self.th.update_querystring('', 'page', 1), 'page=1')
-        self.assertEqual(self.th.update_querystring('page=1', 'page', 2), 'page=2')
-        self.assertEqual(self.th.update_querystring('session=3&page=1', 'page', 2), 'session=3&page=2')
-        self.assertEqual(self.th.update_querystring('page=1&session=3', 'page', 2), 'session=3&page=2')
-        self.assertEqual(self.th.update_querystring('session=3&page=1&ordering=id', 'page', 2), 'session=3&ordering=id&page=2')
-        self.assertEqual(self.th.update_querystring('session=3&ordering=id', 'page', 2), 'session=3&ordering=id&page=2')
+        qs = lambda t: text_type(t).encode('utf8')
+        self.assertEqual(self.th.update_querystring(qs(''), 'page', 1), 'page=1')
+        self.assertEqual(self.th.update_querystring(qs('page=1'), 'page', 2), 'page=2')
+        self.assertEqual(self.th.update_querystring(qs('session=3&page=1'), 'page', 2), 'session=3&page=2')
+        self.assertEqual(self.th.update_querystring(qs('page=1&session=3'), 'page', 2), 'session=3&page=2')
+        self.assertEqual(self.th.update_querystring(qs('session=3&page=1&ordering=id'), 'page', 2), 'session=3&ordering=id&page=2')
+        self.assertEqual(self.th.update_querystring(qs('session=3&ordering=id'), 'page', 2), 'session=3&ordering=id&page=2')
 
     def test_get_verbose_name(self):
         self.assertEqual(self.th.get_verbose_name(User, 'username'), 'Username')
