@@ -11,10 +11,24 @@ import unittest
 
 from flask import g
 
-from flask_peewee.rest import RestAPI, RestResource, Authentication, UserAuthentication
+from flask_peewee.rest import Authentication
+from flask_peewee.rest import RestAPI
+from flask_peewee.rest import RestResource
+from flask_peewee.rest import UserAuthentication
 from flask_peewee.tests.base import FlaskPeeweeTestCase
-from flask_peewee.tests.test_app import User, Message, Note, TestModel, APIKey, AModel, BModel, CModel, EModel, FModel
-from flask_peewee.utils import get_next, make_password, check_password
+from flask_peewee.tests.test_app import AModel
+from flask_peewee.tests.test_app import APIKey
+from flask_peewee.tests.test_app import BModel
+from flask_peewee.tests.test_app import CModel
+from flask_peewee.tests.test_app import EModel
+from flask_peewee.tests.test_app import FModel
+from flask_peewee.tests.test_app import Message
+from flask_peewee.tests.test_app import Note
+from flask_peewee.tests.test_app import TestModel
+from flask_peewee.tests.test_app import User
+from flask_peewee.utils import check_password
+from flask_peewee.utils import get_next
+from flask_peewee.utils import make_password
 
 
 class RestApiTestCase(FlaskPeeweeTestCase):
@@ -26,10 +40,11 @@ class RestApiTestCase(FlaskPeeweeTestCase):
         TestModel.create_table()
 
     def response_json(self, response):
-        return json.loads(response.data)
+        return json.loads(response.data.decode('utf8'))
 
     def auth_headers(self, username, password):
-        return {'Authorization': 'Basic %s' % base64.b64encode('%s:%s' % (username, password))}
+        data = '%s:%s' % (username, password)
+        return {'Authorization': 'Basic %s' % base64.b64encode(data.encode('utf8')).decode('utf8')}
 
     def conv_date(self, dt):
         return dt.strftime('%Y-%m-%d %H:%M:%S')
@@ -194,7 +209,7 @@ class RestApiResourceTestCase(RestApiTestCase):
 
         self.assertEqual(AModel.select().count(), 1)
         a_obj = AModel.get(a_field='ax')
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': a_obj.id,
             'a_field': 'ax',
         })
@@ -209,7 +224,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         a_obj = AModel.get(a_field='ay')
 
         self.assertEqual(b_obj.a, a_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': b_obj.id,
             'b_field': 'by',
             'a': {
@@ -231,7 +246,7 @@ class RestApiResourceTestCase(RestApiTestCase):
 
         self.assertEqual(c_obj.b, b_obj)
         self.assertEqual(b_obj.a, a_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': c_obj.id,
             'c_field': 'cz',
             'b': {
@@ -254,7 +269,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         e_obj = EModel.get(e_field='ey')
 
         self.assertEqual(f_obj.e, e_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': f_obj.id,
             'f_field': 'fy',
             'e': {
@@ -271,7 +286,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         f_obj = FModel.get(f_field='fz')
 
         self.assertEqual(f_obj.e, None)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': f_obj.id,
             'f_field': 'fz',
             'e': None,
@@ -286,7 +301,7 @@ class RestApiResourceTestCase(RestApiTestCase):
 
         self.assertEqual(AModel.select().count(), 2)
         a_obj = AModel.get(id=self.a2.id)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': self.a2.id,
             'a_field': 'a2-xxx',
         })
@@ -301,7 +316,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         a_obj = AModel.get(id=self.a2.id)
 
         self.assertEqual(b_obj.a, a_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': b_obj.id,
             'b_field': 'b2-yyy',
             'a': {
@@ -323,7 +338,7 @@ class RestApiResourceTestCase(RestApiTestCase):
 
         self.assertEqual(c_obj.b, b_obj)
         self.assertEqual(b_obj.a, a_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': c_obj.id,
             'c_field': 'c2-zzz',
             'b': {
@@ -346,7 +361,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         e_obj = EModel.get(id=self.e1.id)
 
         self.assertEqual(f_obj.e, e_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': f_obj.id,
             'f_field': 'f1-yyy',
             'e': {
@@ -363,7 +378,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         f_obj = FModel.get(id=self.f2.id)
 
         self.assertEqual(f_obj.e, None)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': f_obj.id,
             'f_field': 'f2-yyy',
             'e': None,
@@ -383,7 +398,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         a_obj = AModel.get(id=self.a2.id)
 
         self.assertEqual(b_obj.a, a_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': b_obj.id,
             'b_field': 'b2-yyy',
             'a': {
@@ -402,7 +417,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         e_obj = EModel.get(id=self.e1.id)
 
         self.assertEqual(f_obj.e, e_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': f_obj.id,
             'f_field': 'f1-zzz',
             'e': {
@@ -424,7 +439,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         a_obj = AModel.get(id=self.a1.id)
 
         self.assertEqual(b_obj.a, a_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': b_obj.id,
             'b_field': 'b2',
             'a': {
@@ -443,7 +458,7 @@ class RestApiResourceTestCase(RestApiTestCase):
         e_obj = EModel.get(id=self.e2.id)
 
         self.assertEqual(f_obj.e, e_obj)
-        self.assertEqual(json.loads(resp.data), {
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {
             'id': f_obj.id,
             'f_field': 'f2',
             'e': {
@@ -456,21 +471,21 @@ class RestApiResourceTestCase(RestApiTestCase):
         self.create_test_models()
 
         resp = self.post_to('/api/cmodel/%s/delete/' % self.c2.id, {})
-        self.assertEqual(json.loads(resp.data), {'deleted': 1})
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {'deleted': 1})
 
         self.assertEqual(CModel.select().count(), 1)
         self.assertEqual(BModel.select().count(), 2)
         self.assertEqual(AModel.select().count(), 2)
 
         resp = self.post_to('/api/amodel/%s/delete/' % self.a1.id, {})
-        self.assertEqual(json.loads(resp.data), {'deleted': 1})
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {'deleted': 1})
 
         self.assertEqual(CModel.select().count(), 0)
         self.assertEqual(BModel.select().count(), 1)
         self.assertEqual(AModel.select().count(), 1)
 
         resp = self.post_to('/api/emodel/%s/delete/' % self.e1.id, {})
-        self.assertEqual(json.loads(resp.data), {'deleted': 1})
+        self.assertEqual(json.loads(resp.data.decode('utf8')), {'deleted': 1})
 
         self.assertEqual(EModel.select().count(), 1)
         self.assertEqual(FModel.select().count(), 2)
@@ -497,12 +512,10 @@ class RestApiBasicTestCase(RestApiTestCase):
         resp_json = self.response_json(resp)
 
         # verify we have page and link to next page
-        self.assertAPIMeta(resp_json, {
-            'model': 'note',
-            'previous': '',
-            'next': '/api/note/?ordering=id&page=2',
-            'page': 1,
-        })
+        self.assertEqual(resp_json['meta']['model'], 'note')
+        self.assertEqual(resp_json['meta']['previous'], '')
+        self.assertEqual(resp_json['meta']['page'], 1)
+        self.assertTrue('page=2' in resp_json['meta']['next'])
 
         # verify response objects are paginated properly
         self.assertAPINotes(resp_json, notes[:20])
@@ -511,12 +524,10 @@ class RestApiBasicTestCase(RestApiTestCase):
         resp = self.app.get('/api/note/?ordering=id&limit=10')
         resp_json = self.response_json(resp)
 
-        self.assertAPIMeta(resp_json, {
-            'model': 'note',
-            'previous': '',
-            'next': '/api/note/?ordering=id&limit=10&page=2',
-            'page': 1,
-        })
+        self.assertEqual(resp_json['meta']['model'], 'note')
+        self.assertEqual(resp_json['meta']['previous'], '')
+        self.assertEqual(resp_json['meta']['page'], 1)
+        self.assertTrue('page=2' in resp_json['meta']['next'])
 
         # verify response objects are paginated properly
         self.assertAPINotes(resp_json, notes[:10])
@@ -525,12 +536,10 @@ class RestApiBasicTestCase(RestApiTestCase):
         resp = self.app.get(resp_json['meta']['next'])
         resp_json = self.response_json(resp)
 
-        self.assertAPIMeta(resp_json, {
-            'model': 'note',
-            'previous': '/api/note/?ordering=id&limit=10&page=1',
-            'next': '/api/note/?ordering=id&limit=10&page=3',
-            'page': 2,
-        })
+        self.assertEqual(resp_json['meta']['model'], 'note')
+        self.assertEqual(resp_json['meta']['page'], 2)
+        self.assertTrue('page=1' in resp_json['meta']['previous'])
+        self.assertTrue('page=3' in resp_json['meta']['next'])
 
         # verify response objects are paginated properly
         self.assertAPINotes(resp_json, notes[10:20])
@@ -539,12 +548,10 @@ class RestApiBasicTestCase(RestApiTestCase):
         resp = self.app.get(resp_json['meta']['next'])
         resp_json = self.response_json(resp)
 
-        self.assertAPIMeta(resp_json, {
-            'model': 'note',
-            'previous': '/api/note/?ordering=id&limit=10&page=2',
-            'next': '',
-            'page': 3,
-        })
+        self.assertEqual(resp_json['meta']['model'], 'note')
+        self.assertEqual(resp_json['meta']['next'], '')
+        self.assertEqual(resp_json['meta']['page'], 3)
+        self.assertTrue('page=2' in resp_json['meta']['previous'])
 
         # verify response objects are paginated properly
         self.assertAPINotes(resp_json, notes[20:])
@@ -596,6 +603,24 @@ class RestApiBasicTestCase(RestApiTestCase):
         })
         self.assertAPINotes(resp_json, Note.filter(user__in=[self.admin, self.inactive]).order_by(Note.id))
 
+        # do a filter with a negation
+        resp = self.app.get('/api/note/?-user__username=admin&ordering=id')
+        resp_json = self.response_json(resp)
+        self.assertAPINotes(resp_json, Note.filter(user__in=[
+            self.normal, self.inactive]).order_by(Note.id))
+
+        # do a filter with an IN operator and multiple IDs
+        # https://github.com/coleifer/flask-peewee/issues/112
+        resp = self.app.get('/api/note/?id__in=1,2,5')
+        resp_json = self.response_json(resp)
+        self.assertAPINotes(resp_json, Note.filter(id__in=[1,2,5]).order_by(Note.id))
+
+        # also test that the IN operator works with list of strings
+        resp = self.app.get('/api/user/?username__in=admin,normal')
+        resp_json = self.response_json(resp)
+        self.assertAPIUsers(resp_json, User.filter(username__in=['admin', 'normal']).order_by(User.id))
+
+
     def test_filter_with_pagination(self):
         users, notes = self.get_users_and_notes()
         notes = list(self.admin.note_set.order_by(Note.id))
@@ -1105,6 +1130,16 @@ class RestApiKeyAuthTestCase(RestApiTestCase):
             ])
             self.assertAPIMeta(resp_json, {'model': 'testmodel', 'next': '', 'page': 1, 'previous': ''})
 
+    def test_auth_headers(self):
+        with self.flask_app.test_client() as c:
+            resp = c.get('/api/testmodel/', headers={'key': 'k', 'secret': 'foo'})
+            self.assertEqual(resp.status_code, 401)
+            self.assertEqual(g.api_key, None)
+
+            resp = c.get('/api/testmodel/', headers={'key': 'k', 'secret': 's'})
+            self.assertEqual(resp.status_code, 200)
+            self.assertEqual(g.api_key, self.k1)
+
     def test_create(self):
         with self.flask_app.test_client() as c:
             test_data = {'data': 't3'}
